
; Function to compute a non-linear least squares fit for a function with two exponential terms
; 
; input variables:
;       tr: time range
;       probe: either 1, 2, 3 or 4
;       scpot_new: interpolated potential output from density_derivation_ASPOC_ON.pro
;       I_phot: computed photoelectron current from density_derivation_ASPOC_ON.pro
;       show_plot: if eq 1 -> output of average photoelectron curve plot
;       start values: for the computation of the 4 fitting coefficients


;##########################################################################################
;##########################################################################################


function exp_fit_2,  tr, probe, scpot_new, I_phot, show_plot, start_values


  scpot_new1 = scpot_new[5:n_elements(scpot_new)-5]
  current = I_phot[5:n_elements(I_phot)-5]

  ; binning the data:

  bins=((max(scpot_new1)-min(scpot_new1))/101.)*findgen(101)+min(scpot_new1)
  binnedData=dblarr(100)

  bincenter=dblarr(100)
  Error=dblarr(100)

  for i=0,99 do begin
    ;print, i
    elementsInthisBin=where(scpot_new1 gt bins(i) and scpot_new1 lt bins(i+1))
    ;print, elementsInthisBin
    bincenter(i)=(bins(i)+bins(i+1))/2.
    if n_elements(elementsInthisBin) gt 1 then begin
      binnedData(i)=mean(current(elementsInthisBin))
      Error(i)=stddev(current(elementsInthisBin)-binnedData(i))   
    endif
  endfor

  remove, where(binnedData lt 1), bins
  remove, where(binnedData lt 1), Error
  remove, where(binnedData lt 1), binnedData



; define starting values for mpfitfun:
;start = [50D, 1D, 500D, 0.2D]
start = start_values

; set boundaries for the fitting values:

pi = replicate({fixed:0, limited:[0,0], limits:[0.D,0.D]},4)

pi(0).limited(0) = 1     ; lower boundary activated
pi(0).limits(0) = 0.01
pi(2).limited(0) = 1
pi(2).limits(0) = 0.01
pi(3).limited(0) = 1
pi(3).limits(0) = 0.01
;pi(4).limited(0) = 1
;pi(4).limits(0) = 0.1
;pi(5).limited(0) = 1
;pi(5).limits(0) = 0.1
;;
pi(1).limited(1) = 1     ; upper boundary activated
pi(1).limits(1) = 10000
pi(3).limited(1) = 1
pi(3).limits(1) = 10000
;
pi(0).limited(1) = 1      ; upper boundary activated
pi(0).limits(1) = 100000
pi(2).limited(1) = 1
pi(2).limits(1) = 100000

; FIT:


result = MPFITFUN('expfit2', bins, binnedData, stddev(binnedData), start, PARINFO=pi, quiet=quiet)



; coeff as string to be used in plot:
coeff_1 = strtrim(string(result[0]),2)
coeff_2 = strtrim(string(result[1]),2)
coeff_3 = strtrim(string(result[2]),2)
coeff_4 = strtrim(string(result[3]),2)



if show_plot eq 1 then begin
scatter32 = PLOT(scpot_new1, I_phot, $
  LINESTYLE='none', SYMBOL='o',/SYM_FILLED, COLOR='grey', $
  NAME='original data')
scatter32.SYM_SIZE=0.3
scatter32.transparency=80

scatter_binned = ERRORPLOT(bins, binneddata, Error, $
  LINESTYLE='none', SYMBOL='star', /SYM_FILLED, COLOR='black', /OVERPLOT, $
  XTITLE='V !Dsc!N [V]', $
  YTITLE='I!De !N + I!D ASPOC !N [µA]', $
  TITLE='MMS '+probe+': '+tr[0]+' to '+tr[1]+': binned data', $
  NAME='Binned data (with errors)')

; Set some properties
scatter_binned.THICK=2
scatter_binned.ERRORBAR_THICK=1.1
scatter_binned.ERRORBAR_COLOR="black"
scatter_binned.ERRORBAR_CAPSIZE=0.08


scatter2 = PLOT(bins, expfit2(bins, result), $
  LINESTYLE='-', COLOR='green', XRANGE=[2,5.5], $
  name='fit with 2 exp', /OVERPLOT)

scatter2.THICK=2

myText = TEXT(min(scpot_new1)+0.9, max(I_phot)*0.7, 'I!D phot!N ='+coeff_1+' *exp(-V!Dsc!N / '+coeff_2+') + !C!C        '+coeff_3+' *exp(-V!Dsc!N / '+coeff_4+')', FONT_COLOR='green', $
  FONT_SIZE=12, FONT_STYLE='italic', /DATA, TARGET=scatter_binned)

leg = LEGEND(TARGET=[scatter32, scatter2, scatter_binned], POSITION=[5.5,240], $
  /DATA, /AUTO_TEXT_COLOR)

;  scatter2.save, 'C:\Users\dteubenbacher\Desktop\Plots\'+update+'\photocurrent_scpot_fit_scatter_probe'+probe+'_'+date+'.png'
endif

return, result


end